@@ -96,9 +96,17 @@ class RecoveryStrategy:
     Base class strategy is to do no recovery.
     """
 
-    def __init__(self):
-        """Initialize an instance of RecoveryStrategy."""
-        self.recovery_strategy = "none"
+    def __init__(self, variant = ""):
+        """Initialize an instance of RecoveryStrategy.
+
+        Parameters
+        ----------
+        variant
+            Variant of the key type. Currently only "cardano" for ed25519 is supported.
+        """
+        self.recovery_strategy = ""
+        self.variant = variant
+        self.passphrase = ""
 
 
 class RecoveryStrategyBip39(RecoveryStrategy):
@@ -108,17 +116,49 @@ class RecoveryStrategyBip39(RecoveryStrategy):
     Derived from RecoveryStrategy class.
     """
 
-    def __init__(self, passphrase: str):
+    def __init__(self, variant = "", passphrase = ""):
         """Initialize an instance of RecoveryStrategyBip39.
 
         Parameters
         ----------
+        variant
+            Variant of the key type. Currently only "cardano" for ed25519 is supported.
         passphrase
             Passphrase used for bip39 generation. Can be empty string. Must be b64 encoded.
         """
         self.passphrase = passphrase
         self.recovery_strategy = "bip39"
+        self.variant = variant
 
+class RecoveryStrategySlip39(RecoveryStrategy):
+    """The RecoveryStrategySlip39 class definition.
+
+    This class specifies the Slip39 recovery strategy used for wallet generation within Python.
+    Derived from RecoveryStrategy class.
+    """
+
+    def __init__(self, group_count, group_threshold, iteration_exponent, variant = "", passphrase = ""):
+        """Initialize an instance of RecoveryStrategySlip39.
+
+        Parameters
+        ----------
+        group_count
+            Total number of group shares to generate [Max: 14 Groups].
+        group_threshold
+            Number of groups needed to restore a master seed with [threshold <= group_count].
+        iteration_exponent
+            The higher the exponent the more PBKDF2 hashing is done. [Exponent: 0-5]
+        variant
+            Variant of the key type. Currently only "cardano" for ed25519 is supported.
+        passphrase
+            Passphrase used for bip39 generation. Can be empty string. Must be b64 encoded.
+        """
+        self.passphrase = passphrase
+        self.recovery_strategy = "slip39"
+        self.variant = variant
+        self.group_count = group_count
+        self.group_threshold = group_threshold
+        self.iteration_exponent = iteration_exponent
 
 class Zymkey(object):
     """The Zymkey class definition.
@@ -1154,7 +1194,7 @@ class Zymkey(object):
         ----------
         key_type
             This parameter indicates the EC curve type that should be
-            associated with the new key pair. (Curve25519 Not Supported)
+            associated with the new key pair.
         master_gen_key
             The master generator key (bytearray) used in the
             derivation of the child key.
@@ -1163,48 +1203,200 @@ class Zymkey(object):
             is attached to.
         recovery_strategy
             RecoveryStrategy() class that defines what strategy to be used
-            {None, Bip39, Slip39[not supported right now]}.
-            Bip39Strategy->passphrase must be b64 encoded.
+            {None, Bip39, Slip39} are currently supported.
+            RecoveryStrategy->passphrase must be b64 encoded.
 
         Returns
         -------
         TYPE
-            a tuple with the slot and the BIP39 mnemonic if specified
+            the slot the master seed was generated in. 0 for starting slip39 sessions.
         """
         master_gen_key_sz = len(master_gen_key)
         master_gen_key_c_ubyte = (c_ubyte * master_gen_key_sz)(*master_gen_key)
-        bip39 = False
-        bip39_passphrase = ""
-
-        if recovery_strategy.recovery_strategy == "bip39":
-            bip39 = True
-            bip39_passphrase = recovery_strategy.passphrase
 
         bip39_mnemonic = c_void_p()
-        if bip39:
+
+        if recovery_strategy is None:
+            recovery_strategy = RecoveryStrategy()
+
+        if recovery_strategy.recovery_strategy == "bip39":
             mnemonic_ptr = byref(bip39_mnemonic)
         else:
             mnemonic_ptr = POINTER(c_void_p)()
 
         kt = keyTypes[key_type]
-        ret = self._zkGenWalletMasterSeed(
+        if recovery_strategy.recovery_strategy != "slip39":
+            ret = self._zkGenWalletMasterSeed(
+                self._zk_ctx,
+                kt,
+                recovery_strategy.variant.encode("utf-8"),
+                wallet_name.encode("utf-8"),
+                master_gen_key_c_ubyte,
+                master_gen_key_sz,
+                recovery_strategy.passphrase.encode("utf-8"),
+                mnemonic_ptr,
+            )
+        else:
+            ret = self._zkOpenGenSlip39Session(
+                self._zk_ctx,
+                kt,
+                recovery_strategy.variant.encode("utf-8"),
+                wallet_name.encode("utf-8"),
+                master_gen_key_c_ubyte,
+                master_gen_key_sz,
+                recovery_strategy.group_count,
+                recovery_strategy.group_threshold,
+                recovery_strategy.iteration_exponent,
+                recovery_strategy.passphrase.encode("utf-8"),
+            )
+
+        if ret < 0:
+            raise AssertionError("bad return code %d" % ret)
+
+        if recovery_strategy.recovery_strategy == "bip39":
+            mnemonic = cast(bip39_mnemonic, c_char_p)
+            return ret, mnemonic.value.decode("utf-8")
+        elif recovery_strategy.recovery_strategy == "slip39":
+            return ret
+        else:
+            return ret
+
+    def set_gen_slip39_group_info(
+        self,
+        group_index,
+        member_count,
+        member_threshold,
+    ):
+        """Configures the number of members and threshold for the group shares (model >= HSM6).
+
+        This method sets the number of members required for a group share once a slip39 session
+        was opened via gen_wallet_master_seed().
+
+        Parameters
+        ----------
+        group_index
+            This parameter indicates the index of the group share
+            to set the amount of member count/threshold for.
+        member_count
+            The total number of members (mnemonics) in this group share.
+        member_threshold
+            The number of members (mnemonics) needed to reconstruct the group share.
+
+        Returns
+        -------
+        TYPE
+            0 on successful configuration. non-zero for error.
+        """
+        ret = self._zkSetSlip39GroupInfo(self._zk_ctx, group_index, member_count, member_threshold)
+        if (ret < 0):
+            raise AssertionError("bad return code %d" % ret)
+
+    def add_gen_slip39_member_pwd(
+        self,
+        passphrase = "",
+    ):
+        """Generates a new mnemonic_str tied to a slip39 member (model >= HSM6).
+
+        This method generates a new member of a group share. Members can also be
+        passphrase protected. Passphrases are not required to be unique. This
+        function is meant to be called after configuring a group via set_gen_slip39_group_info().
+
+        Parameters
+        ----------
+        passphrase
+            This parameter indicates the passphrase of the slip39 member
+            and is associated with the mnemonic string generated. Can
+            be empty string for no passphrase.
+
+        Returns
+        -------
+        TYPE
+            A 24-word recovery phrase known as a mnemonic sentence. non-zero for error.
+        """
+
+        mnemonic_sentence = c_void_p()
+        mnemonic_ptr = byref(mnemonic_sentence)
+        ret = self._zkAddSlip39Member(self._zk_ctx, passphrase.encode("utf-8"), mnemonic_ptr)
+        mnemonic = cast(mnemonic_sentence, c_char_p)
+        if mnemonic is None:
+            raise AssertionError("Mnemonic returned nothing")
+        return ret, mnemonic.value.decode("utf-8")
+
+    def cancel_slip39_session(self):
+        """Cancels an active slip39 session (model >= HSM6).
+
+        This method cancels an ongoing slip39 session for both master seed generation and recovery.
+
+        Returns
+        -------
+        TYPE
+            0 on success. non-zero for error.
+        """
+
+        ret = self._zkCancelSlip39Session(self._zk_ctx)
+        if (ret < 0):
+            raise AssertionError("bad return code %d" % ret)
+
+    def gen_oversight_wallet(
+        self,
+        key_type,
+        pub_key,
+        chain_code,
+        node_addr,
+        wallet_name,
+        variant = ""
+    ):
+        """Generates a supervisory bip32 wallet. (model >= HSM6).
+
+        This method generates a new supervisory Bip32 wallet. Meant for
+        read-only transactions and supervising history.
+
+        Parameters
+        ----------
+        key_type
+            This parameter indicates the EC curve type that should be
+            associated with the new key pair.
+        pub_key
+            The public key (bytearray) of the last
+            hardened node of the node address.
+        chain_code
+            The chain code (bytearray) of the last
+            hardened node of the node address.
+        node_addr
+            The bip32 node address used. (EX: "m/1852'/1815'/0'").
+        wallet_name
+            The name of the wallet (string) that this master seed
+            is attached to.
+        variant
+            Key type variant to generate from. Currently only "cardano" is
+            supported for "ed25519".
+
+        Returns
+        -------
+        TYPE
+            the slot the oversight wallet was generated in.
+        """
+
+        kt = keyTypes[key_type]
+        pubkey_sz = len(pub_key)
+        pubkey_c_ubyte = (c_ubyte * pubkey_sz)(*pub_key)
+        chaincode_sz = len(chain_code)
+        chaincode_c_ubyte = (c_ubyte * chaincode_sz)(*chain_code)
+        ret = self._zkGenOversightWallet(
             self._zk_ctx,
             kt,
-            wallet_name.encode("utf-8"),
-            master_gen_key_c_ubyte,
-            master_gen_key_sz,
-            bip39_passphrase.encode("utf-8"),
-            mnemonic_ptr,
+            variant.encode("utf-8"),
+            pubkey_c_ubyte,
+            chaincode_c_ubyte,
+            node_addr.encode("utf-8"),
+            wallet_name.encode("utf-8")
         )
         if ret < 0:
             raise AssertionError("bad return code %d" % ret)
-        if bip39:
-            mnemonic = cast(bip39_mnemonic, c_char_p)
-            return ret, mnemonic.value.decode("utf-8")
-        else:
-            return ret, None
 
-    def gen_wallet_child_key(self, parent_key_slot, index, hardened):
+        return ret
+
+    def gen_wallet_child_key(self, parent_key_slot, index, hardened, return_chain_code = False):
         """Generates a child key based on a parent key that is in a wallet (model >= HSM6).
 
         This method generates a child key based on a parent key that is
@@ -1220,23 +1412,38 @@ class Zymkey(object):
             derivation which becomes part of the node address.
         hardened
             If true, the key is a hardened key.
+        return_chain_code
+            If true, returns the chain code for the key as well.
+            (Must be from a hardened key).
 
         Returns
         -------
         TYPE
-            the allocated slot on success
+            the allocated slot on success, or a tuple containing the chain code as well.
         """
-        ret = self._zkGenWalletChildKey(self._zk_ctx, parent_key_slot, index, hardened)
+        chain_code_data = c_void_p()
+        if return_chain_code:
+            chain_ptr = byref(chain_code_data)
+        else:
+            chain_ptr = POINTER(c_void_p)()
+
+        ret = self._zkGenWalletChildKey(self._zk_ctx, parent_key_slot, index, hardened, return_chain_code, chain_ptr)
         if ret < 0:
             raise AssertionError("bad return code %d" % ret)
-        return ret
 
-    def restore_wallet_master_seed_from_bip39_mnemonic(
-        self, key_type, master_gen_key, wallet_name, bip39_passphrase, bip39_mnemonic
+        if return_chain_code:
+            dc = (c_ubyte * 32).from_address(chain_code_data.value)
+            chain_code = bytearray(dc)
+            return ret, chain_code
+        else:
+            return ret
+
+    def restore_wallet_master_seed(
+        self, key_type, master_gen_key, wallet_name, recovery_strategy, mnemonics = None
     ):
-        """Restore a wallet's master seed based on a BIP39 mnemonic string (model >= HSM6).
+        """Restore a wallet's master seed based on the recovery strategy object (model >= HSM6).
 
-        This method restores a wallet's master seed based on a BIP39
+        This method restores a wallet's master seed based on a
         mnemonic string and a master generator key. This method can be
         used in the process of wallet duplication.
 
@@ -1244,16 +1451,18 @@ class Zymkey(object):
         ----------
         key_type
             This parameter indicates the EC curve type that should be
-            associated with the new key pair. (Curve25519 Not Supported)
+            associated with the new key pair.
         master_gen_key
             The master generator key used in the derivation of
             the child key.
         wallet_name
             Name of the new wallet to be generated.
-        bip39_passphrase
-            Passphrase used to generate the bip39_mnemonic.
-        bip39_mnemonic
-            The BIP39 mnemonic string.
+        recovery_strategy
+            RecoveryStategy class object that provides the type of recovery
+            and key variant required for restoration.
+        mnemonics
+            Mnemonic sentences required for restoration, number of mnemonics dependant
+            on recovery strategy used. This field is not used for Slip39.
 
         Returns
         -------
@@ -1264,19 +1473,65 @@ class Zymkey(object):
         master_gen_key_c_ubyte = (c_ubyte * master_gen_key_sz)(*master_gen_key)
 
         kt = keyTypes[key_type]
-        ret = self._zkRestoreWalletMasterSeedFromMnemonic(
-            self._zk_ctx,
-            kt,
-            wallet_name.encode("utf-8"),
-            master_gen_key_c_ubyte,
-            master_gen_key_sz,
-            bip39_passphrase.encode("utf-8"),
-            bip39_mnemonic.encode("utf-8"),
-        )
-        if ret < 0:
+        if recovery_strategy.recovery_strategy == "bip39":
+            if mnemonics is None:
+                raise AssertionError("Bip39 requires a mnemonic sentence")
+            ret = self._zkRestoreWalletMasterSeedFromBip39Mnemonic(
+                self._zk_ctx,
+                kt,
+                recovery_strategy.variant.encode("utf-8"),
+                wallet_name.encode("utf-8"),
+                master_gen_key_c_ubyte,
+                master_gen_key_sz,
+                recovery_strategy.passphrase.encode("utf-8"),
+                mnemonics.encode("utf-8"),
+            )
+        elif recovery_strategy.recovery_strategy == "slip39":
+            ret = self._zkOpenRestoreSlip39Session(
+                self._zk_ctx,
+                kt,
+                recovery_strategy.variant.encode("utf-8"),
+                wallet_name.encode("utf-8"),
+                master_gen_key_c_ubyte,
+                master_gen_key_sz,
+                recovery_strategy.passphrase.encode("utf-8"),
+            )
+        else:
+            raise AssertionError("Not a supported recovery strategy")
+
+        if ret < 0 and recovery_strategy.recovery_strategy != "slip39":
             raise AssertionError("bad return code %d" % ret)
         return ret
 
+    def add_restore_slip39_mnemonic(
+        self,
+        mnemonic_sentence,
+        passphrase = "",
+    ):
+        """Feed a mnemonic string and the passphrase associated with it (model >= HSM6).
+
+        This method feeds in mnemonic sentences (shards) into the module.
+        Meant to be called after starting a restore_wallet_master_seed() slip39 session.
+        Will return -1 until the master seed is reconstructed properly.
+
+        Parameters
+        ----------
+        mnemonic_sentence
+            24-word recovery phrase associated with the slip39 member.
+        passphrase
+            This parameter indicates the passphrase of the slip39 member
+            and is associated with the mnemonic string generated. Can
+            be empty string for no passphrase.
+
+        Returns
+        -------
+        TYPE
+            A -1 for no change in status. Otherwise returns the slot of the master seed successfully
+            reconstructed from the last shard passed in.
+        """
+        ret = self._zkAddRestoreSlip39Mnemonic(self._zk_ctx, passphrase.encode("utf-8"), mnemonic_sentence.encode("utf-8"))
+        return ret
+
     def get_wallet_node_addr(self, slot):
         """Get a wallet node address from a key slot (model >= HSM6)
 
@@ -2134,35 +2389,109 @@ class Zymkey(object):
             POINTER(c_void_p),
         ]
 
-        _zkGenWalletMasterSeed = zkalib.zkGenWalletMasterSeed
+        _zkGenWalletMasterSeed = zkalib.zkGenWalletMasterSeedWithBip39
         _zkGenWalletMasterSeed.restype = c_int
         _zkGenWalletMasterSeed.argtypes = [
             c_void_p,
             c_int,
             c_char_p,
+            c_char_p,
             c_void_p,
             c_int,
             c_char_p,
             POINTER(c_void_p),
         ]
 
-        _zkRestoreWalletMasterSeedFromMnemonic = (
-            zkalib.zkRestoreWalletMasterSeedFromMnemonic
-        )
-        _zkRestoreWalletMasterSeedFromMnemonic.restype = c_int
-        _zkRestoreWalletMasterSeedFromMnemonic.argtypes = [
+        _zkOpenGenSlip39Session = zkalib.zkGenWalletMasterSeedWithSlip39
+        _zkOpenGenSlip39Session.restype = c_int
+        _zkOpenGenSlip39Session.argtypes = [
             c_void_p,
             c_int,
             c_char_p,
+            c_char_p,
+            c_void_p,
+            c_int,
+            c_int,
+            c_int,
+            c_int,
+            c_char_p,
+        ]
+
+        _zkSetSlip39GroupInfo = zkalib.zkSetSlip39GroupInfo
+        _zkSetSlip39GroupInfo.restype = c_int
+        _zkSetSlip39GroupInfo.argtypes = [
+            c_void_p,
+            c_int,
+            c_int,
+            c_int,
+        ]
+
+        _zkAddSlip39Member = zkalib.zkAddSlip39MemberPassword
+        _zkAddSlip39Member.restype = c_int
+        _zkAddSlip39Member.argtypes = [
+            c_void_p,
+            c_char_p,
+            POINTER(c_void_p),
+        ]
+
+        _zkCancelSlip39Session = zkalib.zkCancelSlip39Session
+        _zkCancelSlip39Session.restype = c_int
+        _zkCancelSlip39Session.argtypes = [
+            c_void_p,
+        ]
+
+        _zkGenOversightWallet = zkalib.zkGenOversightWallet
+        _zkGenOversightWallet.restype = c_int
+        _zkGenOversightWallet.argtypes = [
+            c_void_p,
+            c_int,
+            c_char_p,
+            c_void_p,
+            c_void_p,
+            c_char_p,
+            c_char_p,
+        ]
+
+        _zkRestoreWalletMasterSeedFromBip39Mnemonic = (
+            zkalib.zkRestoreWalletMasterSeedFromBip39Mnemonic
+        )
+        _zkRestoreWalletMasterSeedFromBip39Mnemonic.restype = c_int
+        _zkRestoreWalletMasterSeedFromBip39Mnemonic.argtypes = [
+            c_void_p,
+            c_int,
+            c_char_p,
+            c_char_p,
             c_void_p,
             c_int,
             c_char_p,
             c_char_p,
         ]
 
+        _zkAddRestoreSlip39Mnemonic = zkalib.zkAddRestoreSlip39Mnemonic
+        _zkAddRestoreSlip39Mnemonic.restype = c_int
+        _zkAddRestoreSlip39Mnemonic.argtypes = [
+            c_void_p,
+            c_char_p,
+            c_char_p,
+        ]
+
+        _zkOpenRestoreSlip39Session = (
+            zkalib.zkRestoreWalletMasterSeedFromSlip39
+        )
+        _zkOpenRestoreSlip39Session.restype = c_int
+        _zkOpenRestoreSlip39Session.argtypes = [
+            c_void_p,
+            c_int,
+            c_char_p,
+            c_char_p,
+            c_void_p,
+            c_int,
+            c_char_p,
+        ]
+
         _zkGenWalletChildKey = zkalib.zkGenWalletChildKey
         _zkGenWalletChildKey.restype = c_int
-        _zkGenWalletChildKey.argtypes = [c_void_p, c_int, c_uint, c_bool]
+        _zkGenWalletChildKey.argtypes = [c_void_p, c_int, c_uint, c_bool, c_bool, POINTER(c_void_p)]
 
         _zkGetWalletNodeAddrFromKeySlot = zkalib.zkGetWalletNodeAddrFromKeySlot
         _zkGetWalletNodeAddrFromKeySlot.restype = c_int
